//---------------------------------------------------------------------------

#include "double.h"
#include "serial.h"

#define STANDARD_TIMEOUT 10000

//---------------------------------------------------------------------------
#pragma package(smart_init)

static uint8_t CmdBuffer[1024]={0};
static uint16_t TimeOutCounter;

////////////////////////////////////////////////////////////////////////////////
// MsDelay
// Wartezeit in Millisekunden
////////////////////////////////////////////////////////////////////////////////
static void MsDelay(WORD DelayInMilliSeconds)
{
DWORD OldTickCount;
  OldTickCount=GetTickCount();
  while ((OldTickCount + DelayInMilliSeconds) > GetTickCount());
}


///////////////////////////////////////////////////////////////////////////////
// SendCommand
// bertrgt einen Befehl zum Doublestick
// Der Befehl kann bis zu 4 Parameter haben. Nicht vorhandene Parameter werden
// ausgelassen.
//
// Befehlsaufbau:
// Byte 0 Befehl
// Byte 1 LowByte Gesamtlnge
// Byte 2 HighByte Gesamtlnge
//
// LowByte Lnge Parameter 1
// HighByte Lnge Parameter 1
// Parameter 1 (1 - 512 byte)

// ...

// LowByte Lnge Parameter 4
// HighByte Lnge Parameter 4
// Parameter 4 (1 - 512 byte)

// Byte n: 8 Bit XOR Checksumme ber gesamten Befehl

///////////////////////////////////////////////////////////////////////////////
static void SendCommand(uint8_t *pData)
{
uint8_t XOR=0;
uint16_t Count;

  FlushSerialBuffer();

  Count=(uint16_t)(CmdBuffer[1]+256*CmdBuffer[2]);
  Count--;
  while (Count--)
    XOR=XOR^CmdBuffer[Count];
  Count=(uint16_t)(CmdBuffer[1]+256*CmdBuffer[2]);
  CmdBuffer[Count-1]=XOR;

  (void)WriteSerial(CmdBuffer, Count);
}

///////////////////////////////////////////////////////////////////////////////
// GetResult
// Warten maximal 10 Sekunden auf eine Antwort.
// Der Doublestick bricht auch eine Befehlssequenz nach 10 Sekunden ab.
// Das Ergebnis ist gltig, wenn die XOR Checksumme der Antwort stimmt.
///////////////////////////////////////////////////////////////////////////////
static uint8_t GetResult(uint16_t TimeOut)
{
uint8_t Result[3];
DWORD OldTickCount;
  OldTickCount=GetTickCount();
  while ((OldTickCount + TimeOut) > GetTickCount())
  {
    if (GetInQueue() >= 3)
    {
      ReadSerial(Result,3);
      if (((Result[0] ^ Result[1])== Result[2]) && Result[1]!=0)
        return Result[1];
      else
        return false;
    }
  }
  return false;
}

///////////////////////////////////////////////////////////////////////////////
// Mount_
// Datentrger mounten
// Die Karte wird neu initialisiert. Das Filesystem eingelesen.
///////////////////////////////////////////////////////////////////////////////
uint8_t Mount_(void)
{
uint16_t x=0;
  // befehl
  CmdBuffer[x++]=MOUNT;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  // berechnete lnge eintragen
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  // checksumme berechnen und befehl senden
  SendCommand(CmdBuffer);
  // warte maximal STANDARD_TIMEOUT auf antwort
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// UnMount_
// Datentrger unmounten
// Geffnete Dateien werden geschlossen. Karte kann danach entnommen werden.
///////////////////////////////////////////////////////////////////////////////
uint8_t UnMount_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=UNMOUNT;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// CardAvailable_
// Die Karte ist vom USB Port sichtbar
///////////////////////////////////////////////////////////////////////////////
uint8_t CardAvailable_(void)
{
uint16_t x=0;
  // befehl
  CmdBuffer[x++]=CARD_AVAILABLE;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  // berechnete lnge eintragen
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  // checksumme berechnen und befehl senden
  SendCommand(CmdBuffer);
  // warte maximal STANDARD_TIMEOUT auf antwort
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// CardNotAvailable_
// Die Karte ist vom USB Port NICHT sichtbar
///////////////////////////////////////////////////////////////////////////////
uint8_t CardNotAvailable_(void)
{
uint16_t x=0;
  // befehl
  CmdBuffer[x++]=CARD_NOT_AVAILABLE;
  // platzhalter fr gesamtlnge. wird als letzes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  // berechnete lnge eintragen
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  // checksumme berechnen und befehl senden
  SendCommand(CmdBuffer);
  // warte maximal STANDARD_TIMEOUT auf antwort
  return GetResult(STANDARD_TIMEOUT);
}

///////////////////////////////////////////////////////////////////////////////
// GetCardStatus_
// Liest den aktuellen Status der Kartenhalterkontakte
// Bit 0: 1 Karte eingelegt, 0 keine Karte eingelegt
// Bit 1: 1 Karte schreibgeschtzt, 0 Karte nicht schreibgeschtzt
// #define CARD_INSERTED	   (1 << 0)
// #define CARD_WRITE_PROTECT  (1 << 1)
///////////////////////////////////////////////////////////////////////////////
uint8_t GetCardStatus_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=GET_CARD_STATUS;
  // platzhalter fr gesamtlnge. wird als letztes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // plus 1 byte fr xor checksumme
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// FileOpen_
// ffnet die Datei mit dem Namen pName
// Handle: [0..3]
// pName: 8.3 Name von A..Z, 0..9, #, &, %, (, ), -, @
// FileOpenMode: FILEOPENMODE_READONLY, FILEOPENMODE_WRITE, FILEOPENMODE_APPEND
///////////////////////////////////////////////////////////////////////////////
uint8_t FileOpen_(uint8_t Handle, uint8_t *pName, uint8_t FileOpenMode,TimeAndDate_t TimeAndDate)
{
uint16_t x=0;
uint16_t p;
uint8_t Name8Dot3[11];
  memset(Name8Dot3,' ',sizeof(Name8Dot3));

  // Name in einen gltigen 8.3 Name ohne Punkt! z.B. ABCDEFGHTXT
  for (p=0; p<11; p++)
  {
    if (*pName=='.')
    {
      p=7;
      pName++;
    }
    else
      Name8Dot3[p]=(char)toupper(*pName++);
  }

  // command
  CmdBuffer[x++]=OPEN;
  // Platzhalter fr Gesamtlnge. wird als letztes eingetragen
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // laenge filename
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // filename
  CmdBuffer[x++]=Name8Dot3[0];
  CmdBuffer[x++]=Name8Dot3[1];
  CmdBuffer[x++]=Name8Dot3[2];
  CmdBuffer[x++]=Name8Dot3[3];
  CmdBuffer[x++]=Name8Dot3[4];
  CmdBuffer[x++]=Name8Dot3[5];
  CmdBuffer[x++]=Name8Dot3[6];
  CmdBuffer[x++]=Name8Dot3[7];
  CmdBuffer[x++]=Name8Dot3[8];
  CmdBuffer[x++]=Name8Dot3[9];
  CmdBuffer[x++]=Name8Dot3[10];
  // laenge filemode
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=FileOpenMode;

  // laenge time and date
  CmdBuffer[x++]=7;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=TimeAndDate.Second;
  CmdBuffer[x++]=TimeAndDate.Minute;
  CmdBuffer[x++]=TimeAndDate.Hour;
  CmdBuffer[x++]=TimeAndDate.Day;
  CmdBuffer[x++]=TimeAndDate.Month;
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year % 256);
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year / 256);

  // plus 1 byte fr xor checksumme
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// FileClose_
// Handle: [0..3]
///////////////////////////////////////////////////////////////////////////////
uint8_t FileClose_(uint8_t Handle,TimeAndDate_t TimeAndDate)
{
uint16_t x=0;
  CmdBuffer[x++]=CLOSE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;

  // laenge time and date
  CmdBuffer[x++]=7;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=TimeAndDate.Second;
  CmdBuffer[x++]=TimeAndDate.Minute;
  CmdBuffer[x++]=TimeAndDate.Hour;
  CmdBuffer[x++]=TimeAndDate.Day;
  CmdBuffer[x++]=TimeAndDate.Month;
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year % 256);
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// FileRead_
///////////////////////////////////////////////////////////////////////////////
uint16_t FileRead_(uint8_t Handle, uint8_t *pData, uint16_t Count, uint16_t Size)
{
uint16_t x=0;
  CmdBuffer[x++]=READ;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // count laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // count
  CmdBuffer[x++]=(uint8_t)(Count % 256);
  CmdBuffer[x++]=(uint8_t)(Count / 256);
  // size laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // size
  CmdBuffer[x++]=(uint8_t)(Size % 256);
  CmdBuffer[x++]=(uint8_t)(Size / 256);
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);

  // Warten auf angeforderte Daten
  MsDelay(500);
  // Daten auslesen
  return ReadSerial(pData, Count*Size);
  // Die Funktion FileRead_ gibt als Rckgabewert die Anzahl der gelesenen Zeichen
  // zurck. Daher muss die berprfung der Datenblockchecksumme durch die Funktion
  // erfolgen, die FileRead_ aufruft!
}


///////////////////////////////////////////////////////////////////////////////
// FileWrite_
///////////////////////////////////////////////////////////////////////////////
uint16_t FileWrite_(uint8_t Handle, uint8_t *pData, uint16_t Count, uint16_t Size)
{
uint16_t x=0;
uint16_t y;
  CmdBuffer[x++]=WRITE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // count laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // count
  CmdBuffer[x++]=(uint8_t)(Count % 256);
  CmdBuffer[x++]=(uint8_t)(Count / 256);
  // size laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // size
  CmdBuffer[x++]=(uint8_t)(Size % 256);
  CmdBuffer[x++]=(uint8_t)(Size / 256);

  // data laenge
  y=Size*Count;
  CmdBuffer[x++]=y % 256;
  CmdBuffer[x++]=y / 256;

  // data
  while (y--)
  {
    CmdBuffer[x++]=*pData++;
  }

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);

  if (GetResult(STANDARD_TIMEOUT))
    return (Count*Size);
  else
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// FileSeek_
///////////////////////////////////////////////////////////////////////////////
uint8_t FileSeek_(uint8_t Handle, uint16_t Position, uint8_t Whence)
{
uint16_t x=0;
  CmdBuffer[x++]=SEEK;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // handle laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // handle
  CmdBuffer[x++]=Handle;
  // position laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // position
  CmdBuffer[x++]=(uint8_t)(Position % 256);
  CmdBuffer[x++]=(uint8_t)(Position / 256);
  // size whence
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // whence
  CmdBuffer[x++]=(uint8_t)(Whence % 256);
  CmdBuffer[x++]=(uint8_t)(Whence / 256);
  // gesamt laenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// Dir_
///////////////////////////////////////////////////////////////////////////////
uint16_t Dir_(uint8_t *pData)
{
uint16_t x=0;
uint16_t Count;
DWord    InQue;
  // command
  CmdBuffer[x++]=DIR;
  // platzhalter gesamtlaenge
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // gesamt laenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  // solange warten bis keine weiteren Daten mehr einlaufen
  do
  {
    InQue=GetInQueue();
    MsDelay(200);
  }
  while (InQue < GetInQueue());
  // Daten auslesen
  Count=ReadSerial(pData, InQue);
  return Count;
}


///////////////////////////////////////////////////////////////////////////////
// ChangeDirectory_
///////////////////////////////////////////////////////////////////////////////
uint8_t ChangeDirectory_(uint8_t *pName)
{
uint16_t x=0;
  CmdBuffer[x++]=CHANGE_DIRECTORY;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // laenge directoryname
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // filename
  CmdBuffer[x++]=*(pName+0);
  CmdBuffer[x++]=*(pName+1);
  CmdBuffer[x++]=*(pName+2);
  CmdBuffer[x++]=*(pName+3);
  CmdBuffer[x++]=*(pName+4);
  CmdBuffer[x++]=*(pName+5);
  CmdBuffer[x++]=*(pName+6);
  CmdBuffer[x++]=*(pName+7);
  CmdBuffer[x++]=*(pName+8);
  CmdBuffer[x++]=*(pName+9);
  CmdBuffer[x++]=*(pName+10);
  // gesamt laenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// CreateDirectory_
///////////////////////////////////////////////////////////////////////////////
uint8_t CreateDirectory_(uint8_t *pName,TimeAndDate_t TimeAndDate)
{
uint16_t x=0;
  CmdBuffer[x++]=CREATE_DIRECTORY;
  // platzhalter gesamtlaenge
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // laenge directoryname
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // directoryname
  CmdBuffer[x++]=toupper(*(pName+0));
  CmdBuffer[x++]=toupper(*(pName+1));
  CmdBuffer[x++]=toupper(*(pName+2));
  CmdBuffer[x++]=toupper(*(pName+3));
  CmdBuffer[x++]=toupper(*(pName+4));
  CmdBuffer[x++]=toupper(*(pName+5));
  CmdBuffer[x++]=toupper(*(pName+6));
  CmdBuffer[x++]=toupper(*(pName+7));
  CmdBuffer[x++]=' ';
  CmdBuffer[x++]=' ';
  CmdBuffer[x++]=' ';

  // laenge time and date
  CmdBuffer[x++]=7;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=TimeAndDate.Second;
  CmdBuffer[x++]=TimeAndDate.Minute;
  CmdBuffer[x++]=TimeAndDate.Hour;
  CmdBuffer[x++]=TimeAndDate.Day;
  CmdBuffer[x++]=TimeAndDate.Month;
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year % 256);
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year / 256);

  // Gesamtlaenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// RenameFile_
///////////////////////////////////////////////////////////////////////////////
uint8_t RenameFile_(uint8_t *pOldName,uint8_t *pNewName,TimeAndDate_t TimeAndDate)
{
uint16_t x=0;
uint16_t p;
uint8_t OldName8Dot3[11];
uint8_t NewName8Dot3[11];

  memset(OldName8Dot3,' ',sizeof(OldName8Dot3));
  for (p=0; p<11; p++)
  {
    if (*pOldName=='.')
    {
      p=7;
      pOldName++;
    }
    else
      OldName8Dot3[p]=(char)toupper(*pOldName++);
  }

  memset(NewName8Dot3,' ',sizeof(NewName8Dot3));
  for (p=0; p<11; p++)
  {
    if (*pNewName=='.')
    {
      p=7;
      pNewName++;
    }
    else
      NewName8Dot3[p]=(char)toupper(*pNewName++);
  }

  CmdBuffer[x++]=RENAME;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // laenge alter filename
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // filename
  CmdBuffer[x++]=OldName8Dot3[0];
  CmdBuffer[x++]=OldName8Dot3[1];
  CmdBuffer[x++]=OldName8Dot3[2];
  CmdBuffer[x++]=OldName8Dot3[3];
  CmdBuffer[x++]=OldName8Dot3[4];
  CmdBuffer[x++]=OldName8Dot3[5];
  CmdBuffer[x++]=OldName8Dot3[6];
  CmdBuffer[x++]=OldName8Dot3[7];
  CmdBuffer[x++]=OldName8Dot3[8];
  CmdBuffer[x++]=OldName8Dot3[9];
  CmdBuffer[x++]=OldName8Dot3[10];
  // laenge neuer filename
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // filename
  CmdBuffer[x++]=NewName8Dot3[0];
  CmdBuffer[x++]=NewName8Dot3[1];
  CmdBuffer[x++]=NewName8Dot3[2];
  CmdBuffer[x++]=NewName8Dot3[3];
  CmdBuffer[x++]=NewName8Dot3[4];
  CmdBuffer[x++]=NewName8Dot3[5];
  CmdBuffer[x++]=NewName8Dot3[6];
  CmdBuffer[x++]=NewName8Dot3[7];
  CmdBuffer[x++]=NewName8Dot3[8];
  CmdBuffer[x++]=NewName8Dot3[9];
  CmdBuffer[x++]=NewName8Dot3[10];

  // laenge time and date
  CmdBuffer[x++]=7;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=TimeAndDate.Second;
  CmdBuffer[x++]=TimeAndDate.Minute;
  CmdBuffer[x++]=TimeAndDate.Hour;
  CmdBuffer[x++]=TimeAndDate.Day;
  CmdBuffer[x++]=TimeAndDate.Month;
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year % 256);
  CmdBuffer[x++]=(uint8_t)(TimeAndDate.Year / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// RemoveFile_
///////////////////////////////////////////////////////////////////////////////
uint8_t RemoveFile_(uint8_t *pName)
{
uint16_t x=0;
uint16_t p;
uint8_t Name8Dot3[11];

  memset(Name8Dot3,' ',sizeof(Name8Dot3));
  for (p=0; p<11; p++)
  {
    if (*pName=='.')
    {
      p=7;
      pName++;
    }
    else
      Name8Dot3[p]=(char)toupper(*pName++);
  }

  CmdBuffer[x++]=REMOVE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // laenge filename
  CmdBuffer[x++]=11;
  CmdBuffer[x++]=0;
  // filename
  CmdBuffer[x++]=Name8Dot3[0];
  CmdBuffer[x++]=Name8Dot3[1];
  CmdBuffer[x++]=Name8Dot3[2];
  CmdBuffer[x++]=Name8Dot3[3];
  CmdBuffer[x++]=Name8Dot3[4];
  CmdBuffer[x++]=Name8Dot3[5];
  CmdBuffer[x++]=Name8Dot3[6];
  CmdBuffer[x++]=Name8Dot3[7];
  CmdBuffer[x++]=Name8Dot3[8];
  CmdBuffer[x++]=Name8Dot3[9];
  CmdBuffer[x++]=Name8Dot3[10];

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// QuickFormat_
///////////////////////////////////////////////////////////////////////////////
uint8_t QuickFormat_(uint8_t *pName)
{
uint16_t x=0;
uint16_t p;
uint8_t Name8Dot3[11];

  memset(Name8Dot3,' ',sizeof(Name8Dot3));
  for (p=0; p<11; p++)
  {
    if (*pName=='.')
    {
      p=7;
      pName++;
    }
    else
      Name8Dot3[p]=(char)toupper(*pName++);
  }

  Name8Dot3[8]=' ';
  Name8Dot3[9]=' ';
  Name8Dot3[10]=' ';

  CmdBuffer[x++]=FORMAT;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;

  // name size
  CmdBuffer[x++]=8;
  CmdBuffer[x++]=0;

  // data
  for (p=0; p<11; p++)
  {
    CmdBuffer[x++]=Name8Dot3[p];
  }

  // gesamt laenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// InitDisplay_
///////////////////////////////////////////////////////////////////////////////
uint8_t InitDisplay_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=INIT_DISPLAY;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}

///////////////////////////////////////////////////////////////////////////////
// ClearDisplay_
///////////////////////////////////////////////////////////////////////////////
uint8_t ClearDisplay_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=CLEAR_DISPLAY;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}

///////////////////////////////////////////////////////////////////////////////
// DisplayOff_
///////////////////////////////////////////////////////////////////////////////
uint8_t DisplayOnOff_(uint8_t OnOff)
{
uint16_t x=0;
  CmdBuffer[x++]=DISPLAY_ONOFF;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  // onoff laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // OnOff
  CmdBuffer[x++]=OnOff;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}

        // Display_Write(uint8_t CursorPosition, uint8_t *pText, uint8_t Count);

///////////////////////////////////////////////////////////////////////////////
// DisplayWrite_
///////////////////////////////////////////////////////////////////////////////
uint8_t DisplayWrite_(uint8_t CursorPosition, uint8_t Count, uint8_t *pText)
{
uint16_t x=0;
  CmdBuffer[x++]=DISPLAY_WRITE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;

  // Cursorposition laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // Cursorposition
  CmdBuffer[x++]=CursorPosition;

  // Count laenge
  CmdBuffer[x++]=1;
  CmdBuffer[x++]=0;
  // Count
  CmdBuffer[x++]=Count;

  // data size
  CmdBuffer[x++]=(uint8_t)(Count % 256);
  CmdBuffer[x++]=(uint8_t)(Count / 256);

  // data
  while (Count--)
  {
    CmdBuffer[x++]=*pText++;
  }

  // Gesamtlaenge absolut plus crc
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// Connect_
///////////////////////////////////////////////////////////////////////////////
uint8_t Connect_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=CONNECT;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// Disconnect_
///////////////////////////////////////////////////////////////////////////////
uint8_t DisConnect_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=DISCONNECT;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}

///////////////////////////////////////////////////////////////////////////////
// WriteDisable_
///////////////////////////////////////////////////////////////////////////////
uint8_t WriteDisable_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=WRITE_DISABLE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// WriteEnable_
///////////////////////////////////////////////////////////////////////////////
uint8_t WriteEnable_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=WRITE_ENABLE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(STANDARD_TIMEOUT);
}


///////////////////////////////////////////////////////////////////////////////
// Ping_
///////////////////////////////////////////////////////////////////////////////
uint8_t Ping_(void)
{
uint16_t x=0;
  CmdBuffer[x++]=PING;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(1000);
}


///////////////////////////////////////////////////////////////////////////////
// Baudrate_
// BaudRate: Baudrate / 100 z.B. 1152 fr 115200 Baud
///////////////////////////////////////////////////////////////////////////////
uint8_t Baudrate_(uint32_t BaudRate)
{
uint16_t x=0;
  CmdBuffer[x++]=BAUDRATE;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;

  // baudrate laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=(uint8_t)(BaudRate % 256);
  CmdBuffer[x++]=(uint8_t)(BaudRate / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return true;
}


///////////////////////////////////////////////////////////////////////////////
// SetDirection_
// Direction Pin7 - Pin20 setzen. 1 -> Ausgang. 0 -> Eingang
// Pin7 -> Bit0
// Pin8 -> Bit1
// ...
// Pin20 -> Bit13
///////////////////////////////////////////////////////////////////////////////
uint8_t SetDirection_(uint16_t Direction)
{
uint16_t x=0;
  CmdBuffer[x++]=IO_SET_DIRECTION;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;

  // direction laenge
  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  // low baudrate
  CmdBuffer[x++]=(uint8_t)(Direction % 256);
  CmdBuffer[x++]=(uint8_t)(Direction / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(1000);
}


///////////////////////////////////////////////////////////////////////////////
// SetPullUp_
// PullUp Eingnge Pin7 - Pin20 setzen.
// Pin7 -> Bit0
// Pin8 -> Bit1
// ...
// Pin20 -> Bit13
///////////////////////////////////////////////////////////////////////////////
uint8_t SetPullUp_(uint16_t PullUp)
{
uint16_t x=0;
  CmdBuffer[x++]=IO_SET_PULLUP;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;

  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=(uint8_t)(PullUp % 256);
  CmdBuffer[x++]=(uint8_t)(PullUp / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(1000);
}


///////////////////////////////////////////////////////////////////////////////
// SetOutput_
// Status Pin7 - Pin20 setzen. Pin muss zustzlich als Direction out geschaltet werden
// Pin7 -> Bit0
// Pin8 -> Bit1
// ...
// Pin20 -> Bit13
///////////////////////////////////////////////////////////////////////////////
uint8_t SetOutput_(uint16_t Output)
{
uint16_t x=0;
  CmdBuffer[x++]=IO_SET_OUTPUT;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;

  CmdBuffer[x++]=2;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=(uint8_t)(Output % 256);
  CmdBuffer[x++]=(uint8_t)(Output / 256);

  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  return GetResult(1000);
}


///////////////////////////////////////////////////////////////////////////////
// GetInput_
// Status Pin7 - Pin20 auslesen
// Pin7 -> Bit0
// Pin8 -> Bit1
// ...
// Pin20 -> Bit13
///////////////////////////////////////////////////////////////////////////////
uint8_t GetInput_(uint16_t *Data)
{
uint16_t x=0;
int Count;
uint8_t c[2];
  CmdBuffer[x++]=IO_GET_INPUT;
  CmdBuffer[x++]=0;
  CmdBuffer[x++]=0;
  x++;
  CmdBuffer[1]=(uint8_t)(x % 256);
  CmdBuffer[2]=(uint8_t)(x / 256);
  SendCommand(CmdBuffer);
  MsDelay(500);
  ReadSerial(c, 2);
  *Data=c[0]+256*c[1];
  return GetResult(1000);
}


